function map(mode, shortcut, command)
    vim.keymap.set(mode, shortcut, command, { noremap = true, silent = true })
end

vim.g.mapleader = ' '       -- <Leader>
vim.g.maplocalleader = '  ' -- <LocalLeader> (2 spaces)

local format = string.format

-- buffers
map('n', '<Leader>b',   ':buffers<CR>:buffer<Space>')
for key, cmd in pairs({ Left = 'bprevious', Right = 'bnext' }) do
    map('n', format('<M-%s>', key), format(':%s<cr>', cmd))
    map('i', format('<M-%s>', key), format('<esc>:%s<cr>', cmd))
end
-- move buffer without moving cursor
map({ 'n', 'v' }, '<M-Up>',      '<C-y>')
map({ 'n', 'v' }, '<M-Down>',    '<C-e>')

--windows
for key, letter in pairs({ Left = 'h', Down = 'j', Up = 'k', Right = 'l' }) do
    map('n', format('<M-S-%s>', key), format(':wincmd %s<cr>', letter))
    map('i', format('<M-S-%s>', key),  format('<esc>:wincmd %s<cr>', letter))
end

-- remove word
map('n', '<M-BS>',      'db')
vim.cmd([[map! <M-BS> <C-W>]])  -- TODO: figure out how to do that with lua
map('n', '<C-Del>',     'dw')
map('i', '<C-Del>',     '<C-O>dw')

-- remove whitespace around cursor
map({ 'n', 'i' }, '<C-S-Del>',
    function()
        local row, pos_cursor = unpack(vim.api.nvim_win_get_cursor(0))
        pos_cursor = pos_cursor + 1 -- api / lua is off by one
        local line = vim.api.nvim_get_current_line()
        local pos_start, pos_end = 0, 0

        while pos_start ~= nil do
            if pos_start <= pos_cursor and pos_end >= pos_cursor then
                local before = line:sub(1, pos_start - 1)
                local after = line:sub(pos_end + 1)
                local space = ''
                if before:len() > 0 and after:len() > 0 then
                    space = ' '
                end
                vim.api.nvim_set_current_line(before .. space .. after)
                vim.api.nvim_win_set_cursor(0, { row, pos_start - 1 })
                break
            end
            pos_start, pos_end = line:find('%s+', pos_end + 1)
        end
    end
)

-- system clipboard
map('v', '<Leader>y',   '"+y')
map('n', '<Leader>p',   '"+p')

-- toggle between beginning of line and beginning of text
map({ 'n', 'i', 'v' }, '<Home>',
    function()
        local row, col = unpack(vim.api.nvim_win_get_cursor(0))
        if col == 0 then
            local col_new = vim.api.nvim_get_current_line():match('^%s*'):len()
            vim.api.nvim_win_set_cursor(0, { row, col_new })
        else
            vim.api.nvim_win_set_cursor(0, { row, 0 })
        end
    end
)

map('n', '<C-F>', '==') -- re-indent line

-- select text with shift + arrow
for _, key in ipairs({ 'Left', 'Up', 'Down', 'Right' }) do
    map({ 'n', 'i' }, format('<S-%s>', key), format('<Esc>v<%s>', key))
    map({ 'v' }, format('<S-%s>', key), format('<%s>', key))
end

map('n', '<F9>', ':Lexplore 20<cr>')    -- file explorer, 20% width
